@@ -20,16 +20,19 @@
 #define __CHARSET_H__
 
 
-#include <glib.h>
+#include "tmaid.h"
 
 
-#define CHARSETNUM 4
+#define charset_length(ch) (charset->length[(gint)(guchar)(ch)])
+#define charset_width(font,text,size) 										\
+				(charset->length[(gint)*(guchar *)(text)]<=1?1				\
+				:MAX((gdk_text_width((font),(text),							\
+				charset->length[(gint)*(guchar *)(text)])+(size)-1)/(size),1))
 
 
 typedef struct {
 	gchar *name;
 	gint length[256];
-	gint width[256];
 	guchar *space;
 } CHARSET;
 
@@ -37,4 +40,9 @@ typedef struct {
 extern CHARSET *charset,charsetlist[];
 
 
+void charset_set(gchar *lang);
+void charset_init(void);
+void charset_size(GdkFont *font,gint *width,gint *height,gint *ascent);
+
+
 #endif
