@@ -151,9 +151,7 @@ void command_saveas(gpointer callback_data,guint callback_action,
 			delete_edit_file(ptw->file);
 			ptw->file[0]='\0';
 			label=add_edit_file(command_open_file,&ptw->same);
-			gtk_label_set_text(GTK_LABEL(ptw->label),command_open_file);
-			gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
-																child,label);
+			gtk_label_set_text(GTK_LABEL(ptw->label),label);
 			g_free(label);
 			/* ɥ˥塼 */
 			gtk_label_set_text(GTK_LABEL(GTK_BIN(ptw->menu_item)->child),
