@@ -33,6 +33,10 @@ void command_saveas(gpointer callback_data,guint callback_action,
 															GtkWidget *widget);
 void command_reload(gpointer callback_data,guint callback_action,
 															GtkWidget *widget);
+#ifdef USE_GDKPIXBUF
+void command_print(gpointer callback_data,guint callback_action,
+															GtkWidget *widget);
+#endif
 void command_property(gpointer callback_data,guint callback_action,
 															GtkWidget *widget);
 void command_close(gpointer callback_data,guint callback_action,
