@@ -19,6 +19,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "abort.h"
+#include "charset.h"
 #include "file.h"
 #include "other.h"
 #include "signal.h"
@@ -97,8 +98,7 @@ gchar *add_edit_file(gchar *file,gint *same)
 			if (ptw->same<0) {
 				ptw->same=0;
 				label=g_strdup_printf("%s:%d",ptw->file,ptw->same);
-				gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),child,
-																		label);
+				gtk_label_set_text(GTK_LABEL(ptw->label),label);
 				g_free(label);
 			}
 			if (*same<=ptw->same)
@@ -118,8 +118,7 @@ gchar *add_edit_file(gchar *file,gint *same)
 			name1=g_basename(ptw->file);
 			if (strcmp(name0,name1)==0) {
 				if (ptw->same<0)
-					gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
-															child,ptw->file);
+					gtk_label_set_text(GTK_LABEL(ptw->label),ptw->file);
 				break;
 			}
 		}
@@ -157,8 +156,7 @@ void delete_edit_file(gchar *file)
 			ptw=gtk_object_get_user_data(GTK_OBJECT(child));
 			name1=g_basename(ptw->file);
 			if (strcmp(name0,name1)==0)
-				gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),child,
-																		name1);
+				gtk_label_set_text(GTK_LABEL(ptw->label),name1);
 		}
 	} else {
 		/* Ʊ̾ƱѥĴ٤ */
@@ -174,8 +172,7 @@ void delete_edit_file(gchar *file)
 				ptw=gtk_object_get_user_data(GTK_OBJECT(child));
 				if (strcmp(file,ptw->file)==0) {
 					ptw->same=-1;
-					gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
-															child,ptw->file);
+					gtk_label_set_text(GTK_LABEL(ptw->label),ptw->file);
 				}
 			}
 	}
@@ -240,11 +237,25 @@ void open_text_file(TEXTWND *ptw)
 	 RET,TXTɥ													*/
 TEXTWND *open_edit_file(gchar *file)
 {
-	gchar *ext,*p,*label;
-	gint i,leng0,leng1,fontsize,ascent;
+	gchar *ext,*p,*text;
+	gint i,leng0,leng1;
 	TEXTWND *ptw;
-	GtkWidget *table,*sub_menu;
+	GdkPixmap *pixmap;
+	GdkBitmap *mask;
+	GtkWidget *table,*sub_menu,*button,*frame,*hbox;
 	static gint fcount=0;	/* եο */
+	static gchar *xpm[]={
+"8 7 2 1",
+" 	c None",
+".	c #000000",
+"..    ..",
+" ..  .. ",
+"  ....  ",
+"   ..   ",
+"  ....  ",
+" ..  .. ",
+"..    .."};
+
 
 	ptw=g_malloc(sizeof(TEXTWND));
 	ptw->same=-1;
@@ -324,20 +335,13 @@ TEXTWND *open_edit_file(gchar *file)
 	}
 
 	/* ե̾Ĵ٤ */
-	label=add_edit_file(ptw->file,&ptw->same);
+	text=add_edit_file(ptw->file,&ptw->same);
 
 	/* ե */
-	ptw->font=ptw->fontname==NULL?gdk_font_ref(system_font)
-											:gdk_fontset_load(ptw->fontname);
-	ptw->fontascent=ptw->fontsize=0;
-	for (i=0x20;i<=0x7e;i++) {
-		gdk_text_extents(ptw->font,(gchar *)&i,1,NULL,NULL,NULL,&ascent,NULL);
-		if (ptw->fontascent<ascent)
-			ptw->fontascent=ascent;
-		fontsize=(gdk_char_height(ptw->font,i)+1)/2;
-		if (ptw->fontsize<fontsize)
-			ptw->fontsize=fontsize;
-	}
+	if (ptw->fontname==NULL
+						|| (ptw->font=gdk_fontset_load(ptw->fontname))==NULL)
+		ptw->font=gdk_font_ref(system_font);
+	charset_size(ptw->font,&ptw->fontwidth,&ptw->fontheight,&ptw->fontascent);
 
 	/* ɽꥢ */
 	ptw->drawing=gtk_drawing_area_new();
@@ -364,6 +368,11 @@ TEXTWND *open_edit_file(gchar *file)
 	gtk_widget_add_events(ptw->drawing,GDK_FOCUS_CHANGE | GDK_BUTTON_PRESS_MASK
 					| GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
 	GTK_WIDGET_SET_FLAGS(ptw->drawing,GTK_CAN_FOCUS);
+	/* ץ饤ޥꥻ쥯 */
+	gtk_selection_add_targets(ptw->drawing,GDK_SELECTION_PRIMARY,targets,
+									sizeof(targets)/sizeof(GtkTargetEntry));
+	gtk_signal_connect(GTK_OBJECT(ptw->drawing),"selection-get",
+													signal_selection_get,ptw);
 	/* С */
 	ptw->hscroll=gtk_hscrollbar_new(
 							GTK_ADJUSTMENT(gtk_adjustment_new(0,0,0,1,1,1)));
@@ -387,10 +396,23 @@ TEXTWND *open_edit_file(gchar *file)
 						GTK_SIGNAL_FUNC(signal_activate_menu_window),table);
 	gtk_menu_append(GTK_MENU(sub_menu),ptw->menu_item);
 	gtk_widget_show(ptw->menu_item);
+	/* ܥ */
+	button=gtk_button_new();
+	frame=gtk_frame_new(NULL);
+	gtk_button_set_relief(GTK_BUTTON(button),GTK_RELIEF_NONE);
+	pixmap=gdk_pixmap_create_from_xpm_d(window->window,&mask,NULL,xpm);
+	gtk_container_add(GTK_CONTAINER(frame),gtk_pixmap_new(pixmap,mask));
+	gtk_container_add(GTK_CONTAINER(button),frame);
+	gtk_signal_connect(GTK_OBJECT(button),"clicked",signal_clicked,table);
+	/* ٥ */
+	ptw->label=gtk_label_new(text);
+	g_free(text);
 	/* ɽ */
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),table,
-														gtk_label_new(label));
-	g_free(label);
+	hbox=gtk_hbox_new(FALSE,4);
+	gtk_box_pack_start(GTK_BOX(hbox),ptw->label,TRUE,TRUE,0);
+	gtk_box_pack_end(GTK_BOX(hbox),button,FALSE,FALSE,0);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),table,hbox);
+	gtk_widget_show_all(hbox);
 	gtk_widget_show_all(table);
 	gtk_notebook_set_page(GTK_NOTEBOOK(notebook),
 						gtk_notebook_page_num(GTK_NOTEBOOK(notebook),table));
