@@ -234,8 +234,7 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 	st=*start;
 	ed=*end;
 	cursor=ptw->cursor;
-	ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+	ptw->cursor.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 	length0=strlen(find);
 	for (i=0;i<256;i++)
 		if (charset->length[i]>max)
@@ -250,9 +249,9 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 	}
 	if (ed.x>=0) {
 		/* ̤ˤΤȤˤ̵ */
-		i=get_sel_byte(&ptw->start,&ptw->off,&st,&ed,ptw->tab);
+		i=get_sel_bytes(ptw,&st,&ed);
 		if (i<=length0) {
-			cpy_sel_mem(&ptw->start,&ptw->off,&st,&ed,ptw->tab,text);
+			cpy_sel_mem(ptw,&st,&ed,text);
 			text[i]='\0';
 			if (ignorecase)	/* ʸ/ʸ̤ */
 				result=strcmp(find,text)==0;
@@ -264,7 +263,7 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 	}
 	if (ed.x<0) {
 		if (arrow) {
-			ed.x=get_width(&ptw->start,&ptw->off,ptw->max-1,ptw->tab);
+			ed.x=get_width(ptw,ptw->max-1);
 			ed.y=ptw->max-1;/*  */
 		} else {
 			ed.x=ed.y=0;/*  */
@@ -277,8 +276,8 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 	userbreak=TRUE;
 	dialog=abort_dialog(_(""));
 	gtk_grab_add(dialog);
-	ed.x=get_data_pos(&ptw->start,&ptw->off,ed.x,ed.y,ptw->tab,FALSE);
-	st.x=get_data_pos(&ptw->start,&ptw->off,st.x,st.y,ptw->tab,FALSE);
+	ed.x=get_data_pos(ptw,ed.x,ed.y,FALSE);
+	st.x=get_data_pos(ptw,st.x,st.y,FALSE);
 	p=get_line_buf(&ptw->start,&ptw->off,st.y);
 	/* ƱʸӤ뤿ʸ */
 	q=p;
@@ -340,16 +339,14 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 					st.x=0;
 					st.y++;
 				}
-				ptw->select.x=get_screen_pos(&ptw->start,&ptw->off,
-														st.x,st.y,ptw->tab);
+				ptw->select.x=get_screen_pos(ptw,st.x,st.y);
 				ptw->select.y=st.y;
 				if (q->margin && q->length<=data.x) {
 					/* ޡΤȤˤϼιԤƬ */
 					data.x=0;
 					data.y++;
 				}
-				ptw->cursor.x=get_screen_pos(&ptw->start,&ptw->off,
-													data.x,data.y,ptw->tab);
+				ptw->cursor.x=get_screen_pos(ptw,data.x,data.y);
 				ptw->cursor.y=data.y;
 				break;
 			}
@@ -418,16 +415,14 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 					st.x=0;
 					st.y++;
 				}
-				ptw->select.x=get_screen_pos(&ptw->start,&ptw->off,
-														st.x,st.y,ptw->tab);
+				ptw->select.x=get_screen_pos(ptw,st.x,st.y);
 				ptw->select.y=st.y;
 				if (q->margin && q->length<=data.x) {
 					/* ޡΤȤˤϼιԤƬ */
 					data.x=0;
 					data.y++;
 				}
-				ptw->cursor.x=get_screen_pos(&ptw->start,&ptw->off,
-													data.x,data.y,ptw->tab);
+				ptw->cursor.x=get_screen_pos(ptw,data.x,data.y);
 				ptw->cursor.y=data.y;
 				break;
 			}
@@ -449,8 +444,8 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
     gtk_widget_destroy(dialog);
 	if (ptw->select.x>=0) {
 		top=ptw->top;
-		sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-		sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+		sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+		sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 		if (ptw->cursor.x<ptw->top.x)
 			ptw->top.x=ptw->cursor.x;
 		else if (ptw->cursor.x-sx+1>ptw->top.x)
@@ -462,6 +457,7 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 		move_text_window(ptw,&top);
 		clear_sel(ptw,&ptw->select,&ptw->cursor);
 		draw_caret(ptw,&cursor);
-		gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,GDK_CURRENT_TIME);
+		gtk_selection_owner_set(ptw->drawing,GDK_SELECTION_PRIMARY,
+															GDK_CURRENT_TIME);
 	}
 }
