@@ -31,7 +31,7 @@ static gchar *fontname;
 static gint ft_id;
 static GdkColor gdk_color[12];
 static GtkWidget *dialog;
-static GtkWidget *button0,*button3;
+static GtkWidget *button0,*button3,*scroll;
 static GtkWidget *check0,*check1,*check2,*check3,*check4,*check5;
 static GtkWidget *check6,*check7,*check8,*check9,*check10,*check11;
 static GtkWidget *clist,*entry0,*entry1,*label,*spin0,*spin1;
@@ -73,8 +73,6 @@ static void property_dialog_select_row(GtkWidget *widget,
 /* ꥹȥܥå */
 static void property_dialog_unselect_row(GtkWidget *widget,
 				gint row,gint column,GdkEventButton *event,gpointer user_data)
-
-
 {
 	gint i;
 
@@ -182,7 +180,7 @@ static void property_dialog_clicked_font(GtkWidget *widget,gpointer user_data)
 			GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(dialog_font)->cancel_button),
 						"clicked",gtk_widget_destroy,GTK_OBJECT(dialog_font));
 	if (fontname!=NULL)
-	gtk_font_selection_dialog_set_font_name(
+		gtk_font_selection_dialog_set_font_name(
 							GTK_FONT_SELECTION_DIALOG(dialog_font),fontname);
 	gtk_widget_show(dialog_font);
 	gtk_grab_add(dialog_font);
@@ -300,7 +298,7 @@ gboolean property_dialog(FTYPE *ft)
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 	GtkAccelGroup *accel;
-	GtkWidget *button1,*button2,*button4,*scroll;
+	GtkWidget *button1,*button2,*button4;
 	GtkWidget *hbox,*tbox,*vbox;
 	GtkWidget *frame0,*frame1,*frame2,*frame3,*frame4;
 
@@ -311,7 +309,6 @@ gboolean property_dialog(FTYPE *ft)
 	gtk_signal_connect_after(GTK_OBJECT(dialog),"key-press-event",
 							GTK_SIGNAL_FUNC(property_dialog_key_press),NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog),"destroy",gtk_main_quit,NULL);
-	gtk_widget_show(dialog);
 	/* 졼 */
 	accel=gtk_accel_group_new();
 	gtk_accel_group_attach(accel,GTK_OBJECT(dialog));
@@ -415,8 +412,6 @@ gboolean property_dialog(FTYPE *ft)
 	gtk_widget_add_accelerator(check10,"clicked",accel,key,
 									GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
 											|GDK_MOD4_MASK|GDK_MOD5_MASK,0);
-	if (check11!=NULL) {
-	}
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check0),ft->gline);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check1),ft->space);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check2),ft->crlf);
@@ -505,7 +500,7 @@ gboolean property_dialog(FTYPE *ft)
 	for (i=0;i<12;i++) {
 		xpm[1]=g_strdup_printf("X\tc #%02X%02X%02X",
 			ft->color[i].red/256,ft->color[i].green/256,ft->color[i].blue/256);
-		pixmap=gdk_pixmap_create_from_xpm_d(dialog->window,&mask,NULL,xpm);
+		pixmap=gdk_pixmap_create_from_xpm_d(window->window,&mask,NULL,xpm);
 		g_free(xpm[1]);
 		gtk_clist_append(GTK_CLIST(clist),&color_name[i]);
 		gtk_clist_set_pixtext(GTK_CLIST(clist),i,0,
