@@ -17,7 +17,6 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include <gdk/gdkkeysyms.h>
-#include "other.h"
 #include "repinfo.h"
 
 
@@ -51,6 +50,7 @@ static gboolean repinfo_dialog_key_press(GtkWidget *widget,GdkEventKey *event,
 gint repinfo_dialog(REPINFO *ri)
 {
 	gchar *text;
+	guint key;
 	GtkAccelGroup *accel;
 	GtkWidget *button0,*button1,*button2,*button3;
 	GtkWidget *frame0,*frame1,*hbox,*tbox,*vbox;
@@ -61,11 +61,29 @@ gint repinfo_dialog(REPINFO *ri)
 	gtk_signal_connect_after(GTK_OBJECT(dialog),"key-press-event",
 							GTK_SIGNAL_FUNC(repinfo_dialog_key_press),NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog),"destroy",gtk_main_quit,NULL);
+	/* 졼 */
+	accel=gtk_accel_group_new();
+	gtk_accel_group_attach(accel,GTK_OBJECT(dialog));
 	/* ܥ */
-	button0=gtk_button_new_with_label(_("ִ"));
-	button1=gtk_button_new_with_label(_(""));
-	button2=gtk_button_new_with_label(_("٤ִ"));
+	button0=gtk_button_new_with_label(NULL);
+	button1=gtk_button_new_with_label(NULL);
+	button2=gtk_button_new_with_label(NULL);
 	button3=gtk_button_new_with_label(_(""));
+	key=gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button0)->child),
+																_("ִ(_R)"));
+	gtk_widget_add_accelerator(button0,"clicked",accel,key,
+									GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
+											|GDK_MOD4_MASK|GDK_MOD5_MASK,0);
+	key=gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button1)->child),
+																_("(_N)"));
+	gtk_widget_add_accelerator(button1,"clicked",accel,key,
+									GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
+											|GDK_MOD4_MASK|GDK_MOD5_MASK,0);
+	key=gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button2)->child),
+														_("٤ִ(_A)"));
+	gtk_widget_add_accelerator(button2,"clicked",accel,'A',
+									GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
+											|GDK_MOD4_MASK|GDK_MOD5_MASK,0);
 	gtk_signal_connect(GTK_OBJECT(button0),"clicked",
 										repinfo_dialog_clicked,(gpointer)1);
 	gtk_signal_connect(GTK_OBJECT(button1),"clicked",
@@ -78,21 +96,6 @@ gint repinfo_dialog(REPINFO *ri)
 	GTK_WIDGET_SET_FLAGS(button1,GTK_CAN_DEFAULT);
 	GTK_WIDGET_SET_FLAGS(button2,GTK_CAN_DEFAULT);
 	GTK_WIDGET_SET_FLAGS(button3,GTK_CAN_DEFAULT);
-	/* 졼 */
-	accel=gtk_accel_group_new();
-	gtk_accel_group_attach(accel,GTK_OBJECT(dialog));
-	gtk_widget_add_accelerator(button0,"clicked",accel,'R',
-									GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
-											|GDK_MOD4_MASK|GDK_MOD5_MASK,0);
-	set_label_pattern(GTK_BIN(button0)->child,'R');
-	gtk_widget_add_accelerator(button1,"clicked",accel,'N',
-									GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
-											|GDK_MOD4_MASK|GDK_MOD5_MASK,0);
-	set_label_pattern(GTK_BIN(button1)->child,'N');
-	gtk_widget_add_accelerator(button2,"clicked",accel,'A',
-									GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
-											|GDK_MOD4_MASK|GDK_MOD5_MASK,0);
-	set_label_pattern(GTK_BIN(button2)->child,'A');
 	/* ե졼ȥܥå */
 	tbox=gtk_vbox_new(FALSE,0);
 	hbox=gtk_hbox_new(FALSE,0);
