@@ -42,11 +42,11 @@ gboolean signal_timeout(gpointer data)
 	if ((page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))>=0) {
 		ptw=gtk_object_get_user_data(GTK_OBJECT(
 					gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),page)));
-		rc.x=(get_align_pos(&ptw->start,&ptw->off,ptw->cursor.x,ptw->cursor.y,
-									ptw->tab,FALSE)-ptw->top.x)*ptw->fontsize;
-		rc.y=(ptw->cursor.y-ptw->top.y)*ptw->fontsize*2;
-		rc.width=ptw->fontsize;
-		rc.height=ptw->fontsize*2;
+		rc.x=(get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE)-ptw->top.x)
+															*ptw->fontwidth;
+		rc.y=(ptw->cursor.y-ptw->top.y)*ptw->fontheight;
+		rc.width=ptw->fontwidth;
+		rc.height=ptw->fontheight;
 		gtk_widget_draw(ptw->drawing,&rc);
 	}
 	return TRUE;
@@ -165,7 +165,7 @@ void signal_value_changed_vscroll(GtkAdjustment *adjust,TEXTWND *ptw)
 
 	top=ptw->top;
 	ptw->top.y=adjust->value;
-	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
+	sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
 	max=get_width_max(ptw);
 	if (ptw->top.x>max-sx+1)
 		ptw->top.x=MAX(max-sx+1,0);
@@ -179,8 +179,8 @@ gboolean signal_config(GtkWidget *widget,
 {
 	gint max,sx,sy;
 
-	sx=MAX(config->width/ptw->fontsize,1);
-	sy=MAX(config->height/(ptw->fontsize*2),1);
+	sx=MAX(config->width/ptw->fontwidth,1);
+	sy=MAX(config->height/ptw->fontheight,1);
 	if (ptw->top.y>ptw->max-sy)
 		ptw->top.y=MAX(ptw->max-sy,0);
 	max=get_width_max(ptw);
@@ -219,8 +219,7 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 	for (i=0;i<10;i++)
 		gdk_color_alloc(gdk_colormap_get_system(),
 									(ptw->syscol?system_color:ptw->color)+i);
-	cursor.x=get_align_pos(&ptw->start,&ptw->off,ptw->cursor.x,ptw->cursor.y,
-															ptw->tab,FALSE);
+	cursor.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 	cursor.y=ptw->cursor.y;
 	if (ptw->select.x==-1) {
 		/* ϰϤʤȤ */
@@ -242,106 +241,105 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 	if (ptw->gline) {
 		/* å */
 		gdk_gc_set_foreground(gc,(ptw->syscol?system_color:ptw->color)+7);
-		for (x=ptw->fontsize-1;x<widget->allocation.width;x+=ptw->fontsize)
+		for (x=ptw->fontwidth-1;x<widget->allocation.width;x+=ptw->fontwidth)
 			gdk_draw_line(widget->window,gc,x,event->area.y,
 										x,event->area.y+event->area.height);
-		for (y=ptw->fontsize*2-1;y<widget->allocation.height;
-															y+=ptw->fontsize*2)
+		for (y=ptw->fontheight-1;y<widget->allocation.height;
+															y+=ptw->fontheight)
 			gdk_draw_line(widget->window,gc,event->area.x,y,
 										event->area.x+event->area.width,y);
 	}
 	if (ptw->vline) {
 		/*  */
 		gdk_gc_set_foreground(gc,(ptw->syscol?system_color:ptw->color)+7);
-		for (x=(ptw->tab-ptw->top.x%ptw->tab)*ptw->fontsize-1;
-						x<widget->allocation.width;x+=ptw->fontsize*ptw->tab)
+		for (x=(ptw->tab-ptw->top.x%ptw->tab)*ptw->fontwidth-1;
+						x<widget->allocation.width;x+=ptw->fontwidth*ptw->tab)
 			gdk_draw_line(widget->window,gc,x,event->area.y,
 										x,event->area.y+event->area.height);
 	}
 	if (ptw->mline) {
 		/* ޡ */
 		gdk_gc_set_foreground(gc,(ptw->syscol?system_color:ptw->color)+6);
-		x=(ptw->margin-ptw->top.x)*ptw->fontsize;
+		x=(ptw->margin-ptw->top.x)*ptw->fontwidth;
 		gdk_draw_line(widget->window,gc,x,event->area.y,
 										x,event->area.y+event->area.height);
 	}
 
 	for (y=0,p=get_line_buf(&ptw->start,&ptw->off,ptw->top.y);
 								y<event->area.y+event->area.height && p!=NULL;
-												y+=ptw->fontsize*2,p=p->next) {
-		if (y+ptw->fontsize*2<event->area.y)
+												y+=ptw->fontheight,p=p->next) {
+		if (y+ptw->fontheight<event->area.y)
 			continue;
-		x=-ptw->top.x*ptw->fontsize;
+		x=-ptw->top.x*ptw->fontwidth;
 		datapos=0;
 		while (x<=event->area.x+event->area.width && datapos<p->length) {
-			if (x+MAX(ptw->tab,2)*ptw->fontsize<event->area.x) {
+			if (x+MAX(ptw->tab,2)*ptw->fontwidth<event->area.x) {
 				/* ¦ɽʤʬ׻ */
 				if (p->text[datapos]=='\t') {
-					x=(((x/ptw->fontsize+ptw->top.x)/ptw->tab+1)
-										*ptw->tab-ptw->top.x)*ptw->fontsize;
+					x=(((x/ptw->fontwidth+ptw->top.x)/ptw->tab+1)
+										*ptw->tab-ptw->top.x)*ptw->fontwidth;
 					datapos++;
-				} else if (datapos+charset->length[(gint)(guchar)
-												p->text[datapos]]<=p->length) {
+				} else if (datapos+charset_length(p->text[datapos])
+																<=p->length) {
 					/* Ⱦ/ */
-					x+=ptw->fontsize
-							*charset->width[(gint)(guchar)p->text[datapos]];
-					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+					x+=ptw->fontwidth*charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
+					datapos+=charset_length(p->text[datapos]);
 				} else {
 					/* Ⱦ */
-					x+=ptw->fontsize;
+					x+=ptw->fontwidth;
 					datapos++;
 				}
 				continue;
 			}
 			/* ϰ */
 			select=start.x!=-1 && (start.y==end.y?
-								start.y==ptw->top.y+y/(ptw->fontsize*2)
-										&& start.x<=ptw->top.x+x/ptw->fontsize
-										&& ptw->top.x+x/ptw->fontsize<end.x
-								:(start.y<ptw->top.y+y/(ptw->fontsize*2)
-									&& ptw->top.y+y/(ptw->fontsize*2)<end.y)
-									|| (start.y==ptw->top.y+y/(ptw->fontsize*2)
-										&& start.x<=ptw->top.x+x/ptw->fontsize)
-									|| (end.y==ptw->top.y+y/(ptw->fontsize*2)
-										&& ptw->top.x+x/ptw->fontsize<end.x));
+							start.y==ptw->top.y+y/ptw->fontheight
+									&& start.x<=ptw->top.x+x/ptw->fontwidth
+									&& ptw->top.x+x/ptw->fontwidth<end.x
+							:(start.y<ptw->top.y+y/ptw->fontheight
+								&& ptw->top.y+y/ptw->fontheight<end.y)
+								|| (start.y==ptw->top.y+y/ptw->fontheight
+									&& start.x<=ptw->top.x+x/ptw->fontwidth)
+								|| (end.y==ptw->top.y+y/ptw->fontheight
+									&& ptw->top.x+x/ptw->fontwidth<end.x));
 			if (p->text[datapos]=='\t') {
 				/*  */
-				width=(((x/ptw->fontsize+ptw->top.x)/ptw->tab+1)
-										*ptw->tab-ptw->top.x)*ptw->fontsize-x;
+				width=(((x/ptw->fontwidth+ptw->top.x)/ptw->tab+1)
+										*ptw->tab-ptw->top.x)*ptw->fontwidth-x;
 				if (select) {
 					/*  */
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+9);
 					gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
-														width,ptw->fontsize*2);
+														width,ptw->fontheight);
 				}
 				if (ptw->uline) {
 					/*  */
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+5);
-					gdk_draw_line(widget->window,gc,x,y+ptw->fontsize*2-1,
-												x+width,y+ptw->fontsize*2-1);
+					gdk_draw_line(widget->window,gc,x,y+ptw->fontheight-1,
+												x+width,y+ptw->fontheight-1);
 				}
 				if (ptw->code) {
 					/* ʸ */
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+2);
 					gdk_draw_rectangle(widget->window,gc,FALSE,
-										x+ptw->fontsize/6,y+ptw->fontsize*2/3,
-										ptw->fontsize*2/3,ptw->fontsize*2/3);
+										x+ptw->fontwidth/6,y+ptw->fontheight/3,
+										ptw->fontheight/3,ptw->fontheight/3);
 				}
 				datapos++;
-			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
-																<=p->length) {
+			} else if (datapos+charset_length(p->text[datapos])<=p->length) {
 				/* Ⱦ/ */
-				width=ptw->fontsize
-							*charset->width[(gint)(guchar)p->text[datapos]];
+				width=ptw->fontwidth*charset_width(ptw->font,
+											p->text+datapos,ptw->fontwidth);
 				if (select) {
 					/*  */
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+9);
 					gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
-														width,ptw->fontsize*2);
+														width,ptw->fontheight);
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+8);
 				} else {
@@ -351,8 +349,7 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 				if (ptw->space && charset->space!=NULL) {
 					space=FALSE;/* ѥڡȽ */
 					for (i=0;charset->space[i]!=0;i+=charset->space[i]+1)
-						if (charset->length[(gint)(guchar)p->text[datapos]]
-															==charset->space[i]
+						if (charset_length(p->text[datapos])==charset->space[i]
 								&& memcmp(p->text+datapos,charset->space+i+1,
 														charset->space[i])==0)
 							space=TRUE;
@@ -362,23 +359,23 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+3);
 					gdk_draw_rectangle(widget->window,gc,TRUE,x+1,y+1,
-													width-1,ptw->fontsize*2-1);
+													width-1,ptw->fontheight-1);
 				} else {
 					/* ʸ */
 					gdk_draw_text(widget->window,ptw->font,gc,
 										x,y+ptw->fontascent,p->text+datapos,
-							charset->length[(gint)(guchar)p->text[datapos]]);
+											charset_length(p->text[datapos]));
 				}
-				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+				datapos+=charset_length(p->text[datapos]);
 			} else {
 				/* Ⱦ */
-				width=ptw->fontsize;
+				width=ptw->fontwidth;
 				if (select) {
 					/*  */
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+9);
 					gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
-														width,ptw->fontsize*2);
+														width,ptw->fontheight);
 					gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+8);
 				} else {
@@ -391,46 +388,46 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 			}
 			/* åȤ */
 			if (caret && GTK_WIDGET_HAS_FOCUS(widget)
-								&& cursor.x==ptw->top.x+x/ptw->fontsize
-								&& cursor.y==ptw->top.y+y/(ptw->fontsize*2)) {
+								&& cursor.x==ptw->top.x+x/ptw->fontwidth
+								&& cursor.y==ptw->top.y+y/ptw->fontheight) {
 				gdk_gc_set_function(gc,GDK_INVERT);
 				gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
-										ins?2:ptw->fontsize,ptw->fontsize*2);
+										ins?2:ptw->fontwidth,ptw->fontheight);
 				gdk_gc_set_function(gc,GDK_COPY);
 			}
 			x+=width;
 		}
 		if (datapos==p->length && ptw->crlf) {
-			if (start.x>=0 && start.y<=ptw->top.y+y/(ptw->fontsize*2)
-									&& ptw->top.y+y/(ptw->fontsize*2)<end.y) {
+			if (start.x>=0 && start.y<=ptw->top.y+y/ptw->fontheight
+									&& ptw->top.y+y/ptw->fontheight<end.y) {
 				/*  */
 				gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+9);
 				gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
-												ptw->fontsize,ptw->fontsize*2);
+											ptw->fontwidth,ptw->fontheight);
 			}
 			gdk_gc_set_foreground(gc,
 									(ptw->syscol?system_color:ptw->color)+4);
 			if (p->next==NULL) {
 				/* եκǸ */
 				gdk_draw_arc(widget->window,gc,TRUE,
-								x+ptw->fontsize/6,y+ptw->fontsize*2/3,
-								ptw->fontsize*2/3,ptw->fontsize*2/3,0,360*64);
+								x+ptw->fontwidth/6,y+ptw->fontheight/3,
+								ptw->fontheight/3,ptw->fontheight/3,0,360*64);
 			} else {
 				if (p->margin) {
 					/* ޡˤ */
-					pt[0].x=pt[3].x=x+ptw->fontsize/6;
-					pt[0].y=pt[1].y=y+ptw->fontsize*2/3;
-					pt[1].x=pt[2].x=x+ptw->fontsize*5/6;
-					pt[2].y=pt[3].y=y+ptw->fontsize*4/3;
+					pt[0].x=pt[3].x=x+ptw->fontwidth/6;
+					pt[0].y=pt[1].y=y+ptw->fontheight/3;
+					pt[1].x=pt[2].x=x+ptw->fontwidth*5/6;
+					pt[2].y=pt[3].y=y+ptw->fontheight*2/3;
 					length=4;
 				} else {
 					/* ̾β */
-					pt[0].x=x+ptw->fontsize/6;
-					pt[1].x=x+ptw->fontsize*5/6;
-					pt[0].y=pt[1].y=y+ptw->fontsize*2/3;
-					pt[2].x=x+ptw->fontsize/2;
-					pt[2].y=y+ptw->fontsize*4/3;
+					pt[0].x=x+ptw->fontwidth/6;
+					pt[1].x=x+ptw->fontwidth*5/6;
+					pt[0].y=pt[1].y=y+ptw->fontheight/3;
+					pt[2].x=x+ptw->fontwidth/2;
+					pt[2].y=y+ptw->fontheight*2/3;
 					length=3;
 				}
 				gdk_draw_polygon(widget->window,gc,TRUE,pt,length);
@@ -438,11 +435,11 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 		}
 		/* åȤ */
 		if (caret && GTK_WIDGET_HAS_FOCUS(widget)
-								&& cursor.x==ptw->top.x+x/ptw->fontsize
-								&& cursor.y==ptw->top.y+y/(ptw->fontsize*2)) {
+								&& cursor.x==ptw->top.x+x/ptw->fontwidth
+								&& cursor.y==ptw->top.y+y/ptw->fontheight) {
 			gdk_gc_set_function(gc,GDK_INVERT);
 			gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
-										ins?2:ptw->fontsize,ptw->fontsize*2);
+										ins?2:ptw->fontwidth,ptw->fontheight);
 			gdk_gc_set_function(gc,GDK_COPY);
 		}
 	}
@@ -467,17 +464,16 @@ gboolean signal_button_press(GtkWidget *widget,
 
 				cursor=ptw->cursor;
 				select=ptw->select;
-				ptw->cursor.x=ptw->top.x+event->x/ptw->fontsize;
-				ptw->cursor.y=ptw->top.y+event->y/(ptw->fontsize*2);
+				ptw->cursor.x=ptw->top.x+event->x/ptw->fontwidth;
+				ptw->cursor.y=ptw->top.y+event->y/ptw->fontheight;
 				shift=(event->state&GDK_SHIFT_MASK)!=0;
 				if (ptw->cursor.y<0)
 					ptw->cursor.y=0;
 				else if (ptw->cursor.y>ptw->max-1)
 					ptw->cursor.y=ptw->max-1;
-				if (ptw->cursor.x<get_width(&ptw->start,&ptw->off,
-													ptw->cursor.y,ptw->tab))
-					ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+				if (ptw->cursor.x<get_width(ptw,ptw->cursor.y))
+					ptw->cursor.x=get_align_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 				if (ptw->select.x>=0
 									&& (!shift || (ptw->cursor.x==ptw->select.x
 										&& ptw->cursor.y==ptw->select.y))) {
@@ -489,10 +485,10 @@ gboolean signal_button_press(GtkWidget *widget,
 												|| ptw->cursor.y!=cursor.y)) {
 					if (ptw->select.x<0) {
 						/* 򤹤 */
-						ptw->select.x=get_align_pos(&ptw->start,&ptw->off,
-											cursor.x,cursor.y,ptw->tab,FALSE);
+						ptw->select.x=get_align_pos(ptw,
+													cursor.x,cursor.y,FALSE);
 						ptw->select.y=cursor.y;
-						gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+						gtk_selection_owner_set(widget,GDK_SELECTION_PRIMARY,
 															GDK_CURRENT_TIME);
 					}
 					clear_sel(ptw,&cursor,&ptw->cursor);
@@ -509,29 +505,26 @@ gboolean signal_button_press(GtkWidget *widget,
 				DOING *d;
 
 				if (gdk_selection_owner_get(GDK_SELECTION_PRIMARY)
-										==window->window && ptw->select.x>=0) {
+										==widget->window && ptw->select.x>=0) {
 					/* 쥯ΥʡʬΤȤ */
 					/* ϰϤ */
-					length=get_sel_byte(&ptw->start,&ptw->off,
-										&ptw->select,&ptw->cursor,ptw->tab);
+					length=get_sel_bytes(ptw,&ptw->select,&ptw->cursor);
 					text=g_malloc((length+1)*sizeof(gchar));
-					cpy_sel_mem(&ptw->start,&ptw->off,
-									&ptw->select,&ptw->cursor,ptw->tab,text);
+					cpy_sel_mem(ptw,&ptw->select,&ptw->cursor,text);
 					text[length]='\0';
 				}
 				/* åȰư */
 				cursor=ptw->cursor;
 				select=ptw->select;
-				ptw->cursor.x=ptw->top.x+event->x/ptw->fontsize;
-				ptw->cursor.y=ptw->top.y+event->y/(ptw->fontsize*2);
+				ptw->cursor.x=ptw->top.x+event->x/ptw->fontwidth;
+				ptw->cursor.y=ptw->top.y+event->y/ptw->fontheight;
 				if (ptw->cursor.y<0)
 					ptw->cursor.y=0;
 				else if (ptw->cursor.y>ptw->max-1)
 					ptw->cursor.y=ptw->max-1;
-				if (ptw->cursor.x<get_width(&ptw->start,&ptw->off,
-													ptw->cursor.y,ptw->tab))
-					ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+				if (ptw->cursor.x<get_width(ptw,ptw->cursor.y))
+					ptw->cursor.x=get_align_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 				/*  */
 				if (ptw->select.x>=0) {
 					ptw->select.x=-1;
@@ -579,8 +572,8 @@ gboolean signal_button_press(GtkWidget *widget,
 
 				top=ptw->top;
 				ptw->top.y+=event->button==4?-4:4;
-				sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-				sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+				sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+				sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 				if (ptw->top.y<0)
 					ptw->top.y=0;
 				else if (ptw->top.y>ptw->max-sy)
@@ -609,24 +602,21 @@ gboolean signal_button_press(GtkWidget *widget,
 				break;
 			cursor=ptw->cursor;
 			i=0;
-			j=datapos=get_data_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+			j=datapos=get_data_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 			if (datapos>0) {
 				type=g_malloc(datapos*sizeof(gint));
 				/* 򳫻ϰ */
 				while (i<datapos)
-					if (i+charset->length[(gint)(guchar)p->text[i]]<=datapos) {
-						for (k=0;k<charset->length[(gint)(guchar)p->text[i]];
-																		k++)
+					if (i+charset_length(p->text[i])<=datapos) {
+						for (k=0;k<charset_length(p->text[i]);k++)
 							type[i+k]=k+1;
-						i+=charset->length[(gint)(guchar)p->text[i]];
+						i+=charset_length(p->text[i]);
 					} else {
 						type[i++]=1;
 					}
 				if (i<p->length) {
-					length=i+charset->length[(gint)(guchar)p->text[i]]
-																	<=p->length
-								?charset->length[(gint)(guchar)p->text[i]]:1;
+					length=i+charset_length(p->text[i])<=p->length
+												?charset_length(p->text[i]):1;
 					ctype=get_char_type(p->text+i,length);
 					j+=length;
 				} else {
@@ -643,24 +633,22 @@ gboolean signal_button_press(GtkWidget *widget,
 				}
 				g_free(type);
 			} else {
-				length=charset->length[(gint)(guchar)p->text[0]]<=p->length
-								?charset->length[(gint)(guchar)p->text[0]]:1;
+				length=charset_length(p->text[0])<=p->length
+												?charset_length(p->text[0]):1;
 				ctype=get_char_type(p->text,length);
 				j+=length;
 			}
 			/* λ */
 			while (j<p->length) {
-				length=j+charset->length[(gint)(guchar)p->text[j]]
-					<=p->length?charset->length[(gint)(guchar)p->text[j]]:1;
+				length=j+charset_length(p->text[j])<=p->length
+												?charset_length(p->text[j]):1;
 				ct=get_char_type(p->text+j,length);
 				if (ctype!=ct)
 					break;
 				j+=length;
 			}
-			ptw->select.x=get_screen_pos(&ptw->start,&ptw->off,
-													i,ptw->cursor.y,ptw->tab);
-			ptw->cursor.x=get_screen_pos(&ptw->start,&ptw->off,
-													j,ptw->cursor.y,ptw->tab);
+			ptw->select.x=get_screen_pos(ptw,i,ptw->cursor.y);
+			ptw->cursor.x=get_screen_pos(ptw,j,ptw->cursor.y);
 			ptw->select.y=ptw->cursor.y;
 			if (ptw->select.x==ptw->cursor.x) {
 				/*  */
@@ -670,7 +658,7 @@ gboolean signal_button_press(GtkWidget *widget,
 				set_menu_bar(ptw);
 				draw_caret(ptw,&cursor);
 				clear_sel(ptw,&ptw->select,&ptw->cursor);
-				gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+				gtk_selection_owner_set(widget,GDK_SELECTION_PRIMARY,
 															GDK_CURRENT_TIME);
 			}
 		}
@@ -697,8 +685,8 @@ static gboolean signal_motion_notify_draw(TEXTWND *ptw,gint x,gint y,
 	cursor=ptw->cursor;
 	select=ptw->select;
 	top=ptw->top;
-	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+	sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+	sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 	ps.x=x;
 	ps.y=y;
 	if (ps.x<0)
@@ -714,8 +702,7 @@ static gboolean signal_motion_notify_draw(TEXTWND *ptw,gint x,gint y,
 		if (x<0 && ptw->top.x>0)
 			ptw->top.x--;
 		else if (x>ptw->drawing->allocation.width
-								&& ptw->top.x<get_width(&ptw->start,&ptw->off,
-												ptw->cursor.y,ptw->tab)+1-sx)
+							&& ptw->top.x<get_width(ptw,ptw->cursor.y)+1-sx)
 			ptw->top.x++;
 	}
 	if (sy>1) {
@@ -725,24 +712,22 @@ static gboolean signal_motion_notify_draw(TEXTWND *ptw,gint x,gint y,
 			ptw->top.y++;
 	}
 	s=ps.x-((ptw->select.x<0?ptw->cursor.x:ptw->select.x)
-													-ptw->top.x)*ptw->fontsize;
-	if (s<ptw->fontsize/2 || ptw->fontsize<s
+												-ptw->top.x)*ptw->fontwidth;
+	if (s<ptw->fontwidth/2 || ptw->fontwidth<s
 					|| (ptw->select.x<0?ptw->cursor.y:ptw->select.y)
-										!=ps.y/(ptw->fontsize*2)+ptw->top.y) {
+										!=ps.y/ptw->fontheight+ptw->top.y) {
 		if (ptw->select.x<0) {
 			/* 򤹤 */
-			ptw->select.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+			ptw->select.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 			ptw->select.y=ptw->cursor.y;
 		}
-		ptw->cursor.x=ps.x/ptw->fontsize+ptw->top.x;
-		ptw->cursor.y=ps.y/(ptw->fontsize*2)+ptw->top.y;
+		ptw->cursor.x=ps.x/ptw->fontwidth+ptw->top.x;
+		ptw->cursor.y=ps.y/ptw->fontheight+ptw->top.y;
 		if (ptw->cursor.y<0)
 			ptw->cursor.y=0;
 		else if (ptw->cursor.y>ptw->max-1)
 			ptw->cursor.y=ptw->max-1;
-		ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+		ptw->cursor.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 	} else if (ptw->select.x>=0) {
 		/*  */
 		ptw->cursor=ptw->select;
@@ -751,7 +736,8 @@ static gboolean signal_motion_notify_draw(TEXTWND *ptw,gint x,gint y,
 	if (ptw->select.x==ptw->cursor.x && ptw->select.y==ptw->cursor.y)
 		ptw->select.x=-1;/* νȽ꤬ƱȤ */
 	else if (select.x==-1)/* 򤹤 */
-		gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,GDK_CURRENT_TIME);
+		gtk_selection_owner_set(ptw->drawing,GDK_SELECTION_PRIMARY,
+															GDK_CURRENT_TIME);
 	if (ptw->cursor.x!=cursor.x || ptw->cursor.y!=cursor.y
 								|| ptw->top.x!=top.x || ptw->top.y!=top.y) {
 		set_menu_bar(ptw);
@@ -848,8 +834,8 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 
 			cursor=ptw->cursor;
 			top=ptw->top;
-			sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-			sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+			sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+			sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 			alt=(event->state&(GDK_MOD1_MASK|GDK_MOD2_MASK|GDK_MOD3_MASK
 											|GDK_MOD4_MASK|GDK_MOD5_MASK))!=0;
 			ctrl=(event->state&GDK_CONTROL_MASK)!=0;
@@ -860,26 +846,25 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 					ptw->cursor.y=0;
 				break;
 			case GDK_End:
-				ptw->cursor.x=get_width(&ptw->start,&ptw->off,
-						ctrl?ptw->cursor.y=ptw->max-1:ptw->cursor.y,ptw->tab);
+				ptw->cursor.x=get_width(ptw,
+								ctrl?ptw->cursor.y=ptw->max-1:ptw->cursor.y);
 				break;
 			case GDK_H:
 			case GDK_h:
 				if (!alt)
 					goto loop;
 			case GDK_Left:
-				ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+				ptw->cursor.x=get_align_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 				if (ptw->cursor.x>0) {
-					ptw->cursor.x=ctrl?get_move_pos(&ptw->start,&ptw->off,
-									ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE)
-							:get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x-1,ptw->cursor.y,ptw->tab,FALSE);
+					ptw->cursor.x=ctrl?get_move_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE)
+							:get_align_pos(ptw,
+										ptw->cursor.x-1,ptw->cursor.y,FALSE);
 				} else if (ptw->cursor.y>0) {
 					/* ιԤκǸ */
 					ptw->cursor.y--;
-					ptw->cursor.x=get_width(&ptw->start,&ptw->off,
-													ptw->cursor.y,ptw->tab);
+					ptw->cursor.x=get_width(ptw,ptw->cursor.y);
 				}
 				break;
 			case GDK_L:
@@ -887,12 +872,12 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 				if (!alt)
 					goto loop;
 			case GDK_Right:
-				width=get_width(&ptw->start,&ptw->off,ptw->cursor.y,ptw->tab);
+				width=get_width(ptw,ptw->cursor.y);
 				if (ptw->cursor.x<width) {
-					ptw->cursor.x=ctrl?get_move_pos(&ptw->start,&ptw->off,
-									ptw->cursor.x,ptw->cursor.y,ptw->tab,TRUE)
-							:get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x+1,ptw->cursor.y,ptw->tab,TRUE);
+					ptw->cursor.x=ctrl?get_move_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,TRUE)
+							:get_align_pos(ptw,
+										ptw->cursor.x+1,ptw->cursor.y,TRUE);
 				} else if (ptw->cursor.y<ptw->max-1) {
 					/* ιԤκǽ */
 					ptw->cursor.x=0;
@@ -903,26 +888,25 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 			case GDK_less:
 				if (!ctrl)
 					goto loop;
-				ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+				ptw->cursor.x=get_align_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 				if (ptw->cursor.x>0) {
-					ptw->cursor.x=get_move_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+					ptw->cursor.x=get_move_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 				} else if (ptw->cursor.y>0) {
 					/* ιԤκǸ */
 					ptw->cursor.y--;
-					ptw->cursor.x=get_width(&ptw->start,&ptw->off,
-													ptw->cursor.y,ptw->tab);
+					ptw->cursor.x=get_width(ptw,ptw->cursor.y);
 				}
 				break;
 			case GDK_period:
 			case GDK_greater:
 				if (!ctrl)
 					goto loop;
-				width=get_width(&ptw->start,&ptw->off,ptw->cursor.y,ptw->tab);
+				width=get_width(ptw,ptw->cursor.y);
 				if (ptw->cursor.x<width) {
-					ptw->cursor.x=get_move_pos(&ptw->start,&ptw->off,
-									ptw->cursor.x,ptw->cursor.y,ptw->tab,TRUE);
+					ptw->cursor.x=get_move_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,TRUE);
 				} else if (ptw->cursor.y<ptw->max-1) {
 					/* ιԤκǽ */
 					ptw->cursor.x=0;
@@ -994,17 +978,14 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 			} else {
 				if (ptw->select.x<0) {
 					/*  */
-					ptw->select.x=get_align_pos(&ptw->start,&ptw->off,
-											cursor.x,cursor.y,ptw->tab,FALSE);
+					ptw->select.x=get_align_pos(ptw,cursor.x,cursor.y,FALSE);
 					ptw->select.y=cursor.y;
 				}
-				if (ptw->select.y==ptw->cursor.y
-								&& ptw->select.x==get_align_pos(&ptw->start,
-										&ptw->off,ptw->cursor.x,ptw->cursor.y,
-															ptw->tab,FALSE))
+				if (ptw->select.y==ptw->cursor.y && ptw->select.x
+						==get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE))
 					ptw->select.x=-1;/* ϰϤƱȤ */
 				else
-					gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+					gtk_selection_owner_set(widget,GDK_SELECTION_PRIMARY,
 															GDK_CURRENT_TIME);
 			}
 			if (ptw->cursor.x==cursor.x && ptw->cursor.y==cursor.y
@@ -1029,20 +1010,19 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 
 			select=ptw->select;
 			if (ptw->select.x<0) {
-				ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+				ptw->cursor.x=get_align_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 				if (ptw->cursor.x>0) {
 					ptw->select.x=(event->state&GDK_CONTROL_MASK)==0
-							?get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x-1,ptw->cursor.y,ptw->tab,FALSE)
-							:get_move_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+							?get_align_pos(ptw,
+										ptw->cursor.x-1,ptw->cursor.y,FALSE)
+							:get_move_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 					ptw->select.y=ptw->cursor.y;
 				} else if (ptw->cursor.y>0) {
 					/* 2ĤιԤ1Ĥˤ */
 					ptw->select.y=ptw->cursor.y-1;
-					ptw->select.x=get_width(&ptw->start,&ptw->off,
-													ptw->select.y,ptw->tab);
+					ptw->select.x=get_width(ptw,ptw->select.y);
 				} else {
 					break;
 				}
@@ -1082,8 +1062,8 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 				break;
 			select=ptw->select;
 			top=ptw->top;
-			sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-			sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+			sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+			sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
 			if (ptw->cursor.x<ptw->top.x)
 				ptw->top.x=ptw->cursor.x;
 			else if (ptw->cursor.x-sx+1>ptw->top.x)
@@ -1108,15 +1088,13 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 
 			select=ptw->select;
 			if (ptw->select.x<0) {
-				width=get_width(&ptw->start,&ptw->off,ptw->cursor.y,ptw->tab);
-				ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+				width=get_width(ptw,ptw->cursor.y);
+				ptw->cursor.x=get_align_pos(ptw,
+											ptw->cursor.x,ptw->cursor.y,FALSE);
 				if (ptw->cursor.x<width) {
 					ptw->select.x=(event->state&GDK_CONTROL_MASK)==0
-						?get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x+1,ptw->cursor.y,ptw->tab,TRUE)
-						:get_move_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,TRUE);
+						?get_align_pos(ptw,ptw->cursor.x+1,ptw->cursor.y,TRUE)
+						:get_move_pos(ptw,ptw->cursor.x,ptw->cursor.y,TRUE);
 					ptw->select.y=ptw->cursor.y;
 				} else if (ptw->cursor.y<ptw->max-1) {
 					/* 2ĤιԤ1Ĥˤ */
@@ -1152,8 +1130,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 			if (ptw->select.x<0) {
 				if (ptw->max-1<=ptw->cursor.y) {
 					ptw->cursor.x=0;
-					ptw->select.x=get_width(&ptw->start,&ptw->off,
-													ptw->cursor.y,ptw->tab);
+					ptw->select.x=get_width(ptw,ptw->cursor.y);
 					if (ptw->select.x<=0) {
 						if (ptw->cursor.y<=0) {
 							ptw->select.x=-1;
@@ -1177,6 +1154,37 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 		}
 		result=TRUE;
 		break;
+	case GDK_F10:
+		{
+			gint sx,sy;
+			GdkPoint top;
+
+			top=ptw->top;
+			sx=MAX(ptw->drawing->allocation.width/ptw->fontwidth,1);
+			sy=MAX(ptw->drawing->allocation.height/ptw->fontheight,1);
+			if (ptw->cursor.x<ptw->top.x)
+				ptw->top.x=ptw->cursor.x;
+			else if (ptw->cursor.x-sx+1>ptw->top.x)
+				ptw->top.x=ptw->cursor.x-sx+1;
+			if (ptw->cursor.y<ptw->top.y)
+				ptw->top.y=ptw->cursor.y;
+			else if (ptw->cursor.y-sy+1>ptw->top.y)
+				ptw->top.y=ptw->cursor.y-sy+1;
+			move_text_window(ptw,&top);
+		}
+		gtk_widget_set_sensitive(gtk_item_factory_get_widget(
+				item_factory_popup,_("<main>/᤹(U)")),ptw->undo!=NULL);
+		gtk_widget_set_sensitive(gtk_item_factory_get_widget(
+				item_factory_popup,_("<main>/ڤ(T)")),ptw->select.x>=0);
+		gtk_widget_set_sensitive(gtk_item_factory_get_widget(
+				item_factory_popup,_("<main>/ԡ(C)")),ptw->select.x>=0);
+		gtk_widget_set_sensitive(gtk_item_factory_get_widget(
+				item_factory_popup,_("<main>/(L)")),ptw->select.x>=0);
+		gtk_menu_popup(GTK_MENU(
+					gtk_item_factory_get_widget(item_factory_popup,"<main>")),
+											NULL,NULL,NULL,NULL,3,event->time);
+		result=TRUE;
+		break;
 	default:
 		loop:
 		if (event->string!=NULL
@@ -1187,11 +1195,10 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 			GdkPoint select;
 
 			select=ptw->select;
-			ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+			ptw->cursor.x=get_align_pos(ptw,ptw->cursor.x,ptw->cursor.y,FALSE);
 			if (ptw->select.x<0 && !ins) {
-				ptw->select.x=get_align_pos(&ptw->start,&ptw->off,
-								ptw->cursor.x+1,ptw->cursor.y,ptw->tab,TRUE);
+				ptw->select.x=get_align_pos(ptw,
+										ptw->cursor.x+1,ptw->cursor.y,TRUE);
 				if (ptw->select.x<=ptw->cursor.x)
 					ptw->select.x=-1;
 				ptw->select.y=ptw->cursor.y;
@@ -1258,7 +1265,7 @@ void signal_destroy_draw(GtkWidget *widget,TEXTWND *ptw)
 ******************************************************************************/
 void signal_style_set(GtkWidget *widget,GtkStyle *style,gpointer user_data)
 {
-	gint i,j,max,sx,sy,fontsize,ascent;
+	gint i;
 	GList *glist;
 	TEXTWND *ptw;
 
@@ -1286,28 +1293,19 @@ void signal_style_set(GtkWidget *widget,GtkStyle *style,gpointer user_data)
 				/* եѹ */
 				gdk_font_unref(ptw->font);
 				ptw->font=gdk_font_ref(system_font);
-				ptw->fontascent=ptw->fontsize=0;
-				for (j=0x20;j<=0x7e;j++) {
-					gdk_text_extents(ptw->font,(gchar *)&j,1,
-												NULL,NULL,NULL,&ascent,NULL);
-					if (ptw->fontascent<ascent)
-						ptw->fontascent=ascent;
-					fontsize=(gdk_char_height(ptw->font,j)+1)/2;
-					if (ptw->fontsize<fontsize)
-						ptw->fontsize=fontsize;
-				}
+				charset_size(ptw->font,
+							&ptw->fontwidth,&ptw->fontheight,&ptw->fontascent);
 				/* С */
-				sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
-				sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
-				if (ptw->top.y>ptw->max-sy)
-					ptw->top.y=MAX(ptw->max-sy,0);
-				max=get_width_max(ptw);
-				if (ptw->top.x>max-sx+1)
-					ptw->top.x=MAX(max-sx+1,0);
+				ptw->cursor.x=ptw->cursor.y=ptw->top.x=ptw->top.y=0;
+				modify_margin(ptw);
 				set_scroll_bar(ptw->hscroll,signal_value_changed_hscroll,ptw,
-												0,max,sx,ptw->top.x);
+														0,get_width_max(ptw),
+						MAX(ptw->drawing->allocation.width/ptw->fontwidth,1),
+																ptw->top.x);
 				set_scroll_bar(ptw->vscroll,signal_value_changed_vscroll,ptw,
-												0,ptw->max-1,sy,ptw->top.y);
+														0,ptw->max-1,
+						MAX(ptw->drawing->allocation.height/ptw->fontheight,1),
+																ptw->top.y);
 			}
 			gtk_widget_draw(ptw->drawing,NULL);/*  */
 #ifdef USE_XIM
@@ -1364,6 +1362,14 @@ void signal_destroy_menu_history(GtkWidget *widget,gchar *file)
 * ʥ/٥ȴؿ(Ρ)                                             *
 *                                                                             *
 ******************************************************************************/
+void signal_clicked(GtkWidget *widget,GtkWidget *child)
+{
+	if (prompt_close((TEXTWND *)gtk_object_get_user_data(GTK_OBJECT(child))))
+		gtk_notebook_remove_page(GTK_NOTEBOOK(notebook),
+						gtk_notebook_page_num(GTK_NOTEBOOK(notebook),child));
+}
+
+
 void signal_switch_page(GtkNotebook *notebook,GtkNotebookPage *page,
 											gint page_num,gpointer user_data)
 {
@@ -1383,7 +1389,7 @@ void signal_destroy_notebook(GtkWidget *widget,gpointer user_data)
 
 /******************************************************************************
 *                                                                             *
-* ʥ/٥ȴؿ(ᥤ)                                             *
+* ʥ/٥ȴؿ(쥯)                                       *
 *                                                                             *
 ******************************************************************************/
 void signal_drag_data_received(GtkWidget *widget,GdkDragContext *context,
@@ -1416,24 +1422,18 @@ void signal_selection_get(GtkWidget *widget,GtkSelectionData *data,
 {
 	gchar *text=NULL;
 	guchar *compound_text;
-	gint length=0,compound_length,format,page;
-	GdkAtom encoding;
+	gint length=0,compound_length,format;
 	TEXTWND *ptw;
+	GdkAtom encoding;
 
 	if (data->selection==GDK_SELECTION_PRIMARY) {
 		/* ץ饤ޥꥻ쥯 */
-		page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
-		if (page>=0) {
-			ptw=gtk_object_get_user_data(GTK_OBJECT(
-					gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),page)));
-			if (ptw->select.x>=0) {
-				length=get_sel_byte(&ptw->start,&ptw->off,
-										&ptw->select,&ptw->cursor,ptw->tab);
-				text=g_malloc((length+1)*sizeof(gchar));
-				cpy_sel_mem(&ptw->start,&ptw->off,
-									&ptw->select,&ptw->cursor,ptw->tab,text);
-				text[length]='\0';
-			}
+		ptw=user_data;
+		if (ptw->select.x>=0) {
+			length=get_sel_bytes(ptw,&ptw->select,&ptw->cursor);
+			text=g_malloc((length+1)*sizeof(gchar));
+			cpy_sel_mem(ptw,&ptw->select,&ptw->cursor,text);
+			text[length]='\0';
 		}
 	} else if (data->selection==atom_clipboard) {
 		/* åץܡ */
@@ -1466,7 +1466,7 @@ void signal_selection_get(GtkWidget *widget,GtkSelectionData *data,
 void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 												guint time,gpointer user_data)
 {
-	gchar *text=NULL,*str,**list;
+	gchar *text=NULL,*tmp,**list;
 	gint i,count;
 	DOING *d;
 	TEXTWND *ptw;
@@ -1498,9 +1498,9 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 				if (count>0) {
 					text=g_strdup(list[0]);
 					for (i=1;i<count;i++) {
-						str=g_strconcat(text,list[i],NULL);
+						tmp=g_strconcat(text,list[i],NULL);
 						g_free(text);
-						text=str;
+						text=tmp;
 					}
 					gdk_free_text_list(list);
 				}
@@ -1527,6 +1527,11 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 }
 
 
+/******************************************************************************
+*                                                                             *
+* ʥ/٥ȴؿ(ᥤ)                                             *
+*                                                                             *
+******************************************************************************/
 gboolean signal_client_event(GtkWidget *widget,GdkEventClient *event,
 															gpointer user_data)
 {
@@ -1540,7 +1545,7 @@ gboolean signal_client_event(GtkWidget *widget,GdkEventClient *event,
 				break;
 		if (p==NULL) {
 			p=g_malloc(sizeof(COMM));
-			p->str=NULL;
+			p->arg=NULL;
 			p->leng=0;
 			p->init_line=0;
 			p->pid=event->data.l[4];
@@ -1553,17 +1558,17 @@ gboolean signal_client_event(GtkWidget *widget,GdkEventClient *event,
 		}
 		switch (event->data_format) {
 			case 8:
-				p->str=g_realloc(p->str,p->leng+16);
-				memcpy(p->str+p->leng,event->data.b,16);
+				p->arg=g_realloc(p->arg,p->leng+16);
+				memcpy(p->arg+p->leng,event->data.b,16);
 				p->leng+=16;
 				for (i=0;i<16;i++)
 					if (event->data.b[i]=='\0')
 						break;
 				if (i<16) {
-					ptw=open_edit_file(p->str);
+					ptw=open_edit_file(p->arg);
 					if (p->init_line!=0)	/* Ԥ˰ư */
 						jump_operation(ptw,p->init_line);
-					g_free(p->str);
+					g_free(p->arg);
 					if (p->prev!=NULL)
 						p->prev->next=p->next;
 					if (p->next!=NULL)
