@@ -30,6 +30,9 @@
 #include "general.h"
 #include "jump.h"
 #include "other.h"
+#ifdef USE_GDKPIXBUF
+#include "print.h"
+#endif
 #include "profile.h"
 #include "replace.h"
 #include "signal.h"
@@ -42,6 +45,8 @@ gboolean ins=TRUE;					/* TRUE:,FALSE: */
 gboolean newfile=FALSE;				/* TRUE:ե,FALSE:ʤ */
 gboolean second=FALSE;				/* TRUE:,FALSE:¸ */
 gchar *hisfile[10];					/* ե */
+gchar *openpath=NULL;				/* Υѥ */
+gchar *savepath=NULL;				/* ̾դ¸Υѥ */
 gchar *clipboard_text=NULL;			/* åץܡɤΥƥ */
 gint def_width,def_height;			/* ǥեȤΥɥΥ */
 gint ftnum=0;						/* ե륿פο */
@@ -69,6 +74,10 @@ GdkColor system_color[12]={	{0,0x0000,0x0000,0x0000},{0,0xffff,0xffff,0xffff},
 							{0,0x8000,0x8000,0x8000},{0,0xa000,0xa000,0xa000},
 							{0,0xffff,0xffff,0xffff},{0,0x0000,0x0000,0x8000},
 							{0,0x0000,0x0000,0x0000},{0,0x8000,0x8000,0x8000}};
+GtkTargetEntry targets[3]={			/* Text maid󶡲ǽʥ쥯 */
+	{"STRING",       0,TARGET_STRING},
+	{"TEXT",         0,TARGET_TEXT}, 
+	{"COMPOUND_TEXT",0,TARGET_COMPOUND_TEXT}};
 
 
 /******************************************************************************
@@ -79,7 +88,7 @@ GdkColor system_color[12]={	{0,0x0000,0x0000,0x0000},{0,0xffff,0xffff,0xffff},
 int main(int argc,char *argv[])
 {
 	gboolean *arg_files;
-	gchar *arg,*endptr,*str,*section=NULL,*key=NULL,*lang,**charsetname;
+	gchar *arg,*endptr,*str,*section,*key;
 	gint i,j,k,l,m,opt,leng;
 	gint init_line=0;					/* ΥåȰ */
 	COMM *p;
@@ -100,8 +109,12 @@ int main(int argc,char *argv[])
 {N_("/ե(F)/̾Ĥ¸(_A)..."),        NULL,         command_saveas,  0,"<Item>"},
 {N_("/ե(F)/¸Ƥ֤᤹(_R)..."),"<control>R", command_reload,  0,"<Item>"},
 {N_("/ե(F)/Sep1"),                           NULL,         NULL,            0,"<Separator>"},
-{N_("/ե(F)/ץѥƥ(_I)..."),              "<alt>Return",command_property,0,"<Item>"},
+#ifdef USE_GDKPIXBUF
+{N_("/ե(F)/(_P)..."),                    "<control>P", command_print,   0,"<Item>"},
 {N_("/ե(F)/Sep2"),                           NULL,         NULL,            0,"<Separator>"},
+#endif
+{N_("/ե(F)/ץѥƥ(_I)..."),              "<alt>Return",command_property,0,"<Item>"},
+{N_("/ե(F)/Sep3"),                           NULL,         NULL,            0,"<Separator>"},
 {N_("/ե(F)/λ(_X)"),                       "<control>Q", command_exit,    0,"<Item>"},
 {N_("/Խ(_E)"),                                   NULL,         NULL,            0,"<Branch>"},
 {N_("/Խ(E)/᤹(_U)"),                       "<control>Z", command_undo,    0,"<Item>"},
@@ -141,11 +154,6 @@ int main(int argc,char *argv[])
 {N_("/٤(_A)"),                             "<control>A", command_all,     0,"<Item>"}};
 	/* Drag&Drop */
 	GtkTargetEntry drag_types[1]={{"text/uri-list",0,0}};
-	/* Text maid󶡲ǽʥ쥯 */
-	GtkTargetEntry targets[]={
-	{"STRING",       0,TARGET_STRING},
-	{"TEXT",         0,TARGET_TEXT}, 
-	{"COMPOUND_TEXT",0,TARGET_COMPOUND_TEXT}};
 	/* ޥɥ饤  */
 	struct arg_opt {
 		const char *name;
@@ -154,14 +162,15 @@ int main(int argc,char *argv[])
 		const char *doc;
 	};
 	const struct arg_opt arg_opts[]={
-	{"history",'h',"NUM",         N_("ե")},
-	{"line",   'l',"NUM",         N_("ΥåȰ")},
-	{"new",    'n',"ON/OFF",      N_("ե򳫤")},
-	{"size",   's',"WIDTH,HEIGHT",N_("ɥΥ")},
-	{"second", 'x',"ON/OFF",      N_("ɥ򳫤")},
-	{"version",'v',NULL,          N_("С")},
-	{"help",   '?',NULL,          N_("إ")},
-	{NULL,     0,  NULL,          NULL}};
+	{"encoding",'c',"CHARSET",     N_("饯󥳡ǥ")},
+	{"history", 'h',"NUM",         N_("ե")},
+	{"line",    'l',"NUM",         N_("ΥåȰ")},
+	{"new",     'n',"ON/OFF",      N_("ե򳫤")},
+	{"size",    's',"WIDTH,HEIGHT",N_("ɥΥ")},
+	{"second",  'x',"ON/OFF",      N_("ɥ򳫤")},
+	{"version", 'v',NULL,          N_("С")},
+	{"help",    '?',NULL,          N_("إ")},
+	{NULL,      0,  NULL,          NULL}};
 
 	/*  */
 	setlocale(LC_ALL,"");
@@ -169,29 +178,7 @@ int main(int argc,char *argv[])
 	textdomain(PACKAGE);
 	gtk_set_locale();
 	gtk_init(&argc,&argv);
-
-	/* Ķѿ */
-	if ((str=g_getenv("LC_ALL"))!=NULL)
-		while (*str!='\0')
-			if (*str++=='.') {
-				lang=g_strdup(str);
-				for (i=0;lang[i]!='\0';i++)
-					if (lang[i]=='@') {
-						lang[i]='\0';
-						break;
-					}
-				for (i=0;charsetlist[i].name!=NULL;i++) {
-					charsetname=g_strsplit(charsetlist[i].name,",",G_MAXINT);
-					for (j=0;charsetname[j]!=NULL;j++)
-						if (g_strcasecmp(str,charsetname[j])==0) {
-							charset=charsetlist+i;
-							break;
-						}
-					g_strfreev(charsetname);
-				}
-				g_free(lang);
-				break;
-			}
+	charset_init();
 
 	def_width=gdk_screen_width()*2/3;
 	def_height=gdk_screen_height()*2/3;
@@ -219,6 +206,8 @@ int main(int argc,char *argv[])
 		if (!profile_get_value(profile,"tmaid","n_pos",
 				(gpointer)&n_pos,sizeof(gint),PROFILE_VALUE_TYPE_INT))
 			n_pos=GTK_POS_TOP;
+		openpath=profile_get_string(profile,"tmaid","openpath");
+		savepath=profile_get_string(profile,"tmaid","savepath");
 		/* ե */
 		for (i=0;i<history;i++) {
 			key=g_strdup_printf("history%d",i);
@@ -272,6 +261,42 @@ int main(int argc,char *argv[])
 				break;
 		}
 		replace_num=i;
+#ifdef USE_GDKPIXBUF
+		/*  */
+		if (!profile_get_value(profile,"printer","mode",
+								(gpointer)&printer.mode,sizeof(gboolean),
+													PROFILE_VALUE_TYPE_BOOL))
+			printer.mode=TRUE;
+		if ((printer.printer=profile_get_string(profile,"printer",
+															"printer"))==NULL)
+			printer.printer=g_strdup("lpr");
+		if ((printer.file=profile_get_string(profile,"printer","file"))==NULL)
+			printer.file=g_strdup("tmaid.ps");
+		if (!profile_get_value(profile,"printer","width",
+								(gpointer)&printer.width,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT))
+			printer.width=594;
+		if (!profile_get_value(profile,"printer","height",
+								(gpointer)&printer.height,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT))
+			printer.height=840;
+		if (!profile_get_value(profile,"printer","left",
+								(gpointer)&printer.left,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT))
+			printer.left=20;
+		if (!profile_get_value(profile,"printer","top",
+								(gpointer)&printer.top,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT))
+			printer.top=20;
+		if (!profile_get_value(profile,"printer","right",
+								(gpointer)&printer.right,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT))
+			printer.right=20;
+		if (!profile_get_value(profile,"printer","bottom",
+								(gpointer)&printer.bottom,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT))
+			printer.bottom=20;
+#endif
 		/* ե륿 */
 		if (ftnum>0)
 			ftype=g_malloc(ftnum*sizeof(FTYPE));
@@ -352,6 +377,15 @@ int main(int argc,char *argv[])
 			g_free(section);
 		}
 		profile_close(profile);
+#ifdef USE_GDKPIXBUF
+	} else {
+		printer.mode=TRUE;
+		printer.printer=g_strdup("lpr");
+		printer.file=g_strdup("tmaid.ps");
+		printer.width=594;
+		printer.height=840;
+		printer.left=printer.top=printer.right=printer.bottom=20;
+#endif
 	}
 
 	/* ޥɥ饤β */
@@ -394,6 +428,9 @@ int main(int argc,char *argv[])
 						break;
 					}
 				switch (opt) {
+					case 'c':
+						charset_set(arg);
+						break;
 					case 'h':
 						strval(&history,arg,10,TRUE);
 						if (history<0)
@@ -503,8 +540,6 @@ int main(int argc,char *argv[])
 	atom_text=gdk_atom_intern("TEXT",FALSE);
 	atom_textmaid=gdk_atom_intern("Text maid InterProcess communication",
 																		FALSE);
-	gtk_selection_add_targets(window,GDK_SELECTION_PRIMARY,
-							targets,sizeof(targets)/sizeof(GtkTargetEntry));
 	gtk_selection_add_targets(window,atom_clipboard,
 							targets,sizeof(targets)/sizeof(GtkTargetEntry));
 	if (!second)
@@ -539,7 +574,7 @@ int main(int argc,char *argv[])
 	menubar=gtk_item_factory_get_widget(item_factory_menu,"<main>");
 	/* ġС */
 	toolbar=gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_ICONS);
-	for (i=0;i<18;i++)
+	for (i=0;toolbar_items[i].name!=(gpointer)-1;i++)
 		if (toolbar_items[i].name==NULL) {
 			gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 		} else {
@@ -609,13 +644,13 @@ int main(int argc,char *argv[])
 			event.data_format=8;
 			for (i=1;i<argc;i++)
 				if (arg_files[i]) {
-					str=get_full_path(argv[i]);
-					leng=strlen(str)+1;
+					arg=get_full_path(argv[i]);
+					leng=strlen(arg)+1;
 					for (j=0;j<leng;j+=16) {
-						memcpy(event.data.b,str+j,MIN(leng-j,16));
+						memcpy(event.data.b,arg+j,MIN(leng-j,16));
 						gdk_event_send_clientmessage_toall((GdkEvent *)&event);
 					}
-					g_free(str);
+					g_free(arg);
 				}
 			g_free(arg_files);
 			gtk_exit(0);
@@ -676,6 +711,14 @@ int main(int argc,char *argv[])
 				(gpointer)&def_height,sizeof(gint),PROFILE_VALUE_TYPE_INT);
 		profile_set_value(profile,"tmaid","n_pos",
 				(gpointer)&n_pos,sizeof(gint),PROFILE_VALUE_TYPE_INT);
+		if (openpath!=NULL)
+			profile_set_value(profile,"tmaid","openpath",(gpointer)openpath,
+											(strlen(openpath)+1)*sizeof(gchar),
+													PROFILE_VALUE_TYPE_STRING);
+		if (savepath!=NULL)
+			profile_set_value(profile,"tmaid","savepath",(gpointer)savepath,
+											(strlen(savepath)+1)*sizeof(gchar),
+													PROFILE_VALUE_TYPE_STRING);
 		/* ե */
 		for (i=0;i<history && hisfile[i]!=NULL;i++) {
 			key=g_strdup_printf("history%d",i);
@@ -734,7 +777,39 @@ int main(int argc,char *argv[])
 													PROFILE_VALUE_TYPE_STRING);
 			g_free(key);
 		}
+#ifdef USE_GDKPIXBUF
+		/*  */
+		profile_set_value(profile,"printer","mode",
+								(gpointer)&printer.mode,sizeof(gboolean),
+													PROFILE_VALUE_TYPE_BOOL);
+		profile_set_value(profile,"printer","printer",
+									(gpointer)printer.printer,
+									(strlen(printer.printer)+1)*sizeof(gchar),
+													PROFILE_VALUE_TYPE_STRING);
+		profile_set_value(profile,"printer","file",(gpointer)printer.file,
+										(strlen(printer.file)+1)*sizeof(gchar),
+													PROFILE_VALUE_TYPE_STRING);
+		profile_set_value(profile,"printer","width",
+								(gpointer)&printer.width,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT);
+		profile_set_value(profile,"printer","height",
+								(gpointer)&printer.height,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT);
+		profile_set_value(profile,"printer","left",
+								(gpointer)&printer.left,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT);
+		profile_set_value(profile,"printer","top",
+								(gpointer)&printer.top,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT);
+		profile_set_value(profile,"printer","right",
+								(gpointer)&printer.right,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT);
+		profile_set_value(profile,"printer","bottom",
+								(gpointer)&printer.bottom,sizeof(gint),
+													PROFILE_VALUE_TYPE_INT);
+#endif
 		/* ե륿 */
+		section=NULL;
 		for (i=0;i<ftnum;i++) {
 			section=g_strdup_printf("filetype%04d",i);
 			profile_set_value(profile,section,"text",(gpointer)ftype[i].text,
@@ -804,6 +879,13 @@ int main(int argc,char *argv[])
 		profile_close(profile);
 	}
 	gdk_font_unref(system_font);
+	g_free(openpath);
+	g_free(savepath);
+	g_free(clipboard_text);
+#ifdef USE_GDKPIXBUF
+	g_free(printer.printer);
+	g_free(printer.file);
+#endif
 	for (i=0;i<find_num;i++)
 		g_free(find_text[i]);
 	for (i=0;i<replace_num;i++)
@@ -816,9 +898,8 @@ int main(int argc,char *argv[])
 		g_free(ftype[i].fontname);
 	}
 	g_free(ftype);
-	g_free(clipboard_text);
 	while (comm!=NULL) {
-		g_free(comm->str);
+		g_free(comm->arg);
 		p=comm->next;
 		g_free(comm);
 		comm=p;
