@@ -94,14 +94,13 @@ void version_dialog(void)
 	gtk_signal_connect(GTK_OBJECT(dialog),"key-press-event",
 							GTK_SIGNAL_FUNC(version_dialog_key_press),NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog),"destroy",gtk_main_quit,NULL);
-	gtk_widget_show(dialog);
 	/* ܥ */
 	button=gtk_button_new_with_label(_("λ"));
 	gtk_signal_connect_object(GTK_OBJECT(button),"clicked",
 										gtk_widget_destroy,GTK_OBJECT(dialog));
 	GTK_WIDGET_SET_FLAGS(button,GTK_CAN_DEFAULT);
 	/* ԥޥå */
-	pixmap=gdk_pixmap_create_from_xpm_d(dialog->window,&mask,NULL,tmaid_xpm);
+	pixmap=gdk_pixmap_create_from_xpm_d(window->window,&mask,NULL,tmaid_xpm);
 
 	/* ե졼ȥܥå */
 	vbox=gtk_vbox_new(FALSE,0);
